const exec = require('execa')
const getStream = require('get-stream')
const tmp = require('tmp-promise')
const fs = require('fs-extra')

const { getDefinitionOptions } = require('./kubernetes')

module.exports = {
  build,
  push
}

async function getFakeDockerfile(config) {
  const { rootDir } = config
  const tmpFile = await tmp.file({
    mode: 0644,
    postfix: ''
  })

  const { base, ports, env } = getDefinitionOptions(config)

  if (!base) {
    throw new Error('missing `base` option for docker build')
  }

  const buildpack = base.split(':')[0]
  const buildpackCmds = getCommandsForBuildpack(buildpack)
  const envEntries = Object.keys(env || {}).map(key => ({
    key,
    value: env[key]
  }))

  const fakeDockerfileContent = `
FROM ${base}

${buildpackCmds}

${envEntries.map(entry => `ENV ${entry.key}=${entry.value}`).join('\n')}

${ports.map(port => `EXPOSE ${port}`).join('\n')}
`.trim()

  if (config.logLevel === 'debug') {
    console.log(`\nFaked dockerfile:\n${fakeDockerfileContent}\n`)
  }

  try {
    await fs.writeFile(tmpFile.path, fakeDockerfileContent)

    return tmpFile
  } catch (err) {
    throw err
  }
}

const getDockerBuildArgs = async config => {
  if (!config.options || !config.options.base) {
    return []
  }

  try {
    const fakeDockerfile = await getFakeDockerfile(config)

    return ['-f', fakeDockerfile.path]
  } catch (err) {
    throw err
  }
}

async function build(config) {
  const { rootDir, tag } = config

  const images = getImages(config)
  const fileArgs = await getDockerBuildArgs(config)

  try {
    const stream = exec('docker', ['build', ...images, rootDir, ...fileArgs])
      .stdout

    // DEBUG: view what's going on in docker build
    // TODO: create summarized version of what's happening per
    // layer and stream into the "Building docker image" spinner
    //
    // stream.pipe(process.stdout)

    await getStream(stream)

    return {
      // TODO: this should probably be the docker image id
      buildId: tag
    }
  } catch (err) {
    return {
      err
    }
  }
}

const defaultPushCmd = name => ['docker', ['push', name]]

function getPushCmd(config) {
  const { tag, name, registry = '' } = getDefinitionOptions(config)

  if (typeof registry !== 'string') {
    return defaultPushCmd(name)
  }

  return defaultPushCmd(registry + '/' + name + ':' + tag)
}

async function push(config) {
  const { stdout, stderr } = await exec(...getPushCmd(config))

  if (stderr) {
    throw stderr
  }
}

function getImages(config = {}) {
  const { registry = '', name, tag } = getDefinitionOptions(config)

  const images = []

  images.push({ name, tag })

  if (registry) {
    images.push({ name: registry + '/' + name, tag })
    images.push({ name: registry + '/' + name, tag: 'latest' })
  }

  return images.reduce((acc, i) => {
    acc.push('-t')
    acc.push([i.name, i.tag].join(':'))

    return acc
  }, [])
}

const cmdsForBuildpack = {
  node: () =>
    `
RUN mkdir /app

COPY . /app
WORKDIR /app

RUN npm install --production

CMD [ "npm", "start" ]`.trim(),
  python: () =>
    `
WORKDIR /usr/src/app

COPY requirements.txt ./
RUN pip install --no-cache-dir -r requirements.txt

COPY . .

CMD [ "python", "main.py" ]`.trim()
}

function getCommandsForBuildpack(buildpack) {
  if (!cmdsForBuildpack[buildpack]) {
    // TEMPORARY: until we add support for more languages
    console.error(new Error('funk.docker.generate: unknown base image'))
    return ''
  }

  return cmdsForBuildpack[buildpack]()
}
