const exec = require('execa')

const kubernetes = require('./kubernetes')

module.exports = {
  applyCharts,
  // Check if helm is installed locally
  checkForHelm,
  // Check if helm is installed on the cluster
  checkForTiller,
  // Check if a certain chart has been installed on the cluster
  checkForChart,
  // Install the chart only if it doesn't exist
  installChartIfNotExists
}

async function applyCharts(config) {
  const { charts } = config

  const chartEntries = Object.keys(charts).map(key => {
    const chart = charts[key]
    // TODO: lookup if should be stable or incubator
    const chartPkg = key.includes('/') ? key : `stable/${key}`

    return {
      name: [config.name, key].join('-'),
      ...chart,
      _pkg: chartPkg
    }
  })

  const results = await Promise.all(
    chartEntries.map(async chart => {
      return await installChartIfNotExists(config, chart)
    })
  )

  return results
}

async function checkForHelm() {
  try {
    await exec('helm', ['version'])

    return true
  } catch (err) {
    return false
  }
}

async function checkForTiller() {
  try {
    const { stdout } = await exec('helm', ['version'])

    return /Server: /.test(stdout)
  } catch (err) {
    return false
  }
}

async function checkForChart(config, chart) {
  try {
    const { services } = await kubernetes.listServices(config, {
      name: chart.name,
      label: 'release',
      output: 'json'
    })

    return services.length > 0
  } catch (err) {
    // TODO: handle error
    console.error(err)
    return false
  }
}

async function installChartIfNotExists(config, chart) {
  try {
    const doesChartExist = await checkForChart(config, chart)

    if (doesChartExist) {
      return { chart, installed: false, alreadyExists: true }
    }

    const { values = {} } = chart
    const chartValues = Object.keys(values)
      .map(key => `${key}=${values[key]}`)
      .join(',')

    const { stdout, stderr } = await exec('helm', [
      'install',
      chart._pkg,
      '--name',
      chart.name,
      '--set',
      chartValues
    ])

    if (config.logLevel === 'debug') {
      console.log(stdout)
    }

    return { chart, installed: true }
  } catch (err) {
    // TODO: handle error
    throw err
  }
}
